@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tk.h,v 1.74.2.13 2005/11/27 02:44:25 das Exp $
+ * RCS: @(#) $Id: tk.h,v 1.74.2.16 2006/05/04 13:09:46 dgp Exp $
  */
 
 #ifndef _TK
@@ -48,21 +48,10 @@ extern "C" {
 #define TK_MAJOR_VERSION   8
 #define TK_MINOR_VERSION   4
 #define TK_RELEASE_LEVEL   TCL_FINAL_RELEASE
-#define TK_RELEASE_SERIAL  12
+#define TK_RELEASE_SERIAL  14
 
 #define TK_VERSION	"8.4"
-#define TK_PATCH_LEVEL	"8.4.12"
-
-/*
- * The following definitions set up the proper options for Macintosh
- * compilers.  We use this method because there is no autoconf equivalent.
- */
-
-#if defined(MAC_TCL) || defined(MAC_OSX_TK)
-#   ifndef REDO_KEYSYM_LOOKUP
-#	define REDO_KEYSYM_LOOKUP
-#   endif
-#endif
+#define TK_PATCH_LEVEL	"8.4.14"
 
 #ifndef _TCL
 #   include <tcl.h>
