@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkPanedWindow.c,v 1.13.2.8 2005/05/31 05:22:06 hobbs Exp $
+ * RCS: @(#) $Id: tkPanedWindow.c,v 1.13.2.10 2006/10/17 05:38:47 dgp Exp $
  */
 
 #include "tkPort.h"
@@ -164,7 +164,7 @@ static void	PanedWindowEventProc _ANSI_ARGS_((ClientData clientData,
 static void	ProxyWindowEventProc _ANSI_ARGS_((ClientData clientData,
 			XEvent *eventPtr));
 static void	DisplayProxyWindow _ANSI_ARGS_((ClientData clientData));
-void		PanedWindowWorldChanged _ANSI_ARGS_((ClientData instanceData));
+static void	PanedWindowWorldChanged _ANSI_ARGS_((ClientData instanceData));
 static int	PanedWindowWidgetObjCmd _ANSI_ARGS_((ClientData clientData,
 			Tcl_Interp *, int objc, Tcl_Obj * CONST objv[]));
 static void	PanedWindowLostSlaveProc _ANSI_ARGS_((ClientData clientData,
@@ -349,7 +349,7 @@ Tk_PanedWindowObjCmd(clientData, interp, objc, objv)
     }
 
     tkwin = Tk_CreateWindowFromPath(interp, Tk_MainWindow(interp), 
-	    Tcl_GetStringFromObj(objv[1], NULL), (char *) NULL);
+	    Tcl_GetStringFromObj(objv[1], NULL), NULL);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
@@ -1225,7 +1225,7 @@ ConfigurePanedWindow(interp, pwPtr, objc, objv)
  *----------------------------------------------------------------------
  */
 
-void
+static void
 PanedWindowWorldChanged(instanceData)
     ClientData instanceData;	/* Information about the paned window. */
 {
