@@ -10,11 +10,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXButton.c,v 1.2.2.9 2005/05/14 20:53:31 das Exp $
+ * RCS: @(#) $Id: tkMacOSXButton.c,v 1.2.2.14 2006/05/26 21:43:05 hobbs Exp $
  */
 
-#include "tkButton.h"
 #include "tkMacOSXInt.h"
+#include "tkButton.h"
+#include "tkMacOSXFont.h"
 #include "tkMacOSXDebug.h"
 
 #define DEFAULT_USE_TK_TEXT 0
@@ -27,8 +28,6 @@
 #define DEF_INSET_TOP 2
 #define DEF_INSET_BOTTOM 4
 
-#include <Carbon/Carbon.h>
-
 /*
  * Some defines used to control what type of control is drawn.
  */
@@ -109,9 +108,6 @@ static void SetupBevelButton _ANSI_ARGS_((MacButton *butPtr,
         ControlRef controlHandle, 
         GWorldPtr destPort, GC gc, Pixmap pixmap));
 
-extern int TkFontGetFirstTextLayout(Tk_TextLayout layout, Tk_Font * font, char * dst); 
-extern void TkMacOSXInitControlFontStyle(Tk_Font tkfont,ControlFontStylePtr fsPtr);
-
 /*
  * The class procedure table for the button widgets.
  */
@@ -204,7 +200,7 @@ TkpDisplayButton(
     int borderWidth;
     Pixmap pixmap;
     int wasUsingControl;
-    int imageWidth, imageHeight;
+    int imageWidth = 0, imageHeight = 0;
     int imageXOffset = 0, imageYOffset = 0; /* image information that will
 					     * be used to restrict disabled
 					     * pixmap as well */
@@ -789,7 +785,7 @@ TkpDestroyButton(
  *----------------------------------------------------------------------
  */
 
-int
+static int
 TkMacOSXInitControl (
     MacButton *mbPtr,                /* Mac button. */
     GWorldPtr  destPort,
@@ -833,12 +829,16 @@ TkMacOSXInitControl (
         controlReference );
         
     if (!mbPtr->userPane) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"Failed to create user pane control\n");
+#endif
         return 1;
     }
     
     if ((status = EmbedControl(mbPtr->userPane,rootControl)) != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"Failed to embed user pane control %d\n", status);
+#endif
         return 1;
     }
     
@@ -857,12 +857,16 @@ TkMacOSXInitControl (
         controlReference );
         
     if (!mbPtr->control) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"failed to create control of type %d\n",procID);
+#endif
         return 1;
     }
     
     if (EmbedControl(mbPtr->control,mbPtr->userPane) != noErr ) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"failed to embed control of type %d\n",procID);
+#endif
         return 1;
     }
     
@@ -985,7 +989,9 @@ TkMacOSXDrawControl(
             TkMacOSXInitControlFontStyle(font, &fontStyle);
             if (bcmp(&mbPtr->fontStyle, &fontStyle, sizeof(fontStyle)) ) {
                 if (SetControlFontStyle(mbPtr->control, &fontStyle) != noErr) {
+#ifdef TK_MAC_DEBUG
                     fprintf(stderr,"SetControlFontStyle failed\n");
+#endif
                 }
                 bcopy(&fontStyle, &mbPtr->fontStyle, 
                         sizeof(fontStyle));
@@ -1003,33 +1009,39 @@ TkMacOSXDrawControl(
     } else {
      SetControlValue(mbPtr->control, 0);
     }
-    
+
     if (!Tk_MacOSXIsAppInFront() || butPtr->state == STATE_DISABLED) {
-        HiliteControl(mbPtr->control, kControlInactivePart);
-    } else if (butPtr->state == STATE_ACTIVE) {
-        if (mbPtr->params.isBevel) {
-           HiliteControl(mbPtr->control, kControlButtonPart);
-        } else {
-            switch (butPtr->type) {
-                case TYPE_BUTTON:
-                    HiliteControl(mbPtr->control,  kControlButtonPart);
-                    break;
-                case TYPE_RADIO_BUTTON:
-                    HiliteControl(mbPtr->control, kControlRadioButtonPart);
-                    break;
-                case TYPE_CHECK_BUTTON:
-                    HiliteControl(mbPtr->control, kControlCheckBoxPart);
-                    break;
-            }
-        }
+	HiliteControl(mbPtr->control, kControlInactivePart);
     } else {
-        HiliteControl(mbPtr->control, kControlNoPart);
+	/*
+	 * Use NoPart for normal and to ensure correct direct transition from
+	 * disabled to active -state. [Bug 706446]
+	 */
+	HiliteControl(mbPtr->control, kControlNoPart);
+
+	if (butPtr->state == STATE_ACTIVE) {
+	    if (mbPtr->params.isBevel) {
+		HiliteControl(mbPtr->control, kControlButtonPart);
+	    } else {
+		switch (butPtr->type) {
+		    case TYPE_BUTTON:
+			HiliteControl(mbPtr->control,  kControlButtonPart);
+			break;
+		    case TYPE_RADIO_BUTTON:
+			HiliteControl(mbPtr->control, kControlRadioButtonPart);
+			break;
+		    case TYPE_CHECK_BUTTON:
+			HiliteControl(mbPtr->control, kControlCheckBoxPart);
+			break;
+		}
+	    }
+	}
     }
     UpdateControlColors(mbPtr);
-        
-    if ((butPtr->type == TYPE_BUTTON) ) {
+
+    if (butPtr->type == TYPE_BUTTON) {
         Boolean isDefault;
-        
+
         if (butPtr->defaultState == STATE_ACTIVE) {
             isDefault = true;
         } else {
@@ -1037,7 +1049,7 @@ TkMacOSXDrawControl(
         }
         if ((err=SetControlData(mbPtr->control, kControlNoPart, 
                 kControlPushButtonDefaultTag,
-                sizeof(isDefault), (Ptr) &isDefault)) != noErr ) {
+                sizeof(isDefault), (Ptr) &isDefault)) != noErr) {
         }
     }
 
@@ -1049,10 +1061,10 @@ TkMacOSXDrawControl(
         SetControlVisibility(mbPtr->control, true, true);
         Draw1Control(mbPtr->userPane);
     }
- 
+
     if (mbPtr->params.isBevel) {
         KillPicture(mbPtr->bevelButtonContent.u.picture);
-    }         
+    }
 }
 
 /*
@@ -1113,7 +1125,9 @@ SetupBevelButton(
      
     if (!(mbPtr->bevelButtonContent.u.picture 
             = OpenCPicture(&mbPtr->picParams)) ) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"OpenCPicture failed\n");
+#endif
     }
     tkPictureIsOpen = 1;
     
@@ -1142,8 +1156,10 @@ SetupBevelButton(
             kControlBevelButtonContentTag,
             sizeof(ControlButtonContentInfo),
             (char *) &mbPtr->bevelButtonContent)) != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,
                 "SetControlData BevelButtonContent failed, %d\n", err );
+#endif
     }
             
     if (butPtr->anchor == TK_ANCHOR_N) {
@@ -1170,8 +1186,10 @@ SetupBevelButton(
             kControlBevelButtonGraphicAlignTag,
             sizeof(ControlButtonGraphicAlignment),
             (char *) &theAlignment)) != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,
                 "SetControlData BevelButtonGraphicAlign failed, %d\n", err );
+#endif
     }
 
     if (butPtr->compound != COMPOUND_NONE) {
@@ -1190,8 +1208,10 @@ SetupBevelButton(
                 kControlBevelButtonTextPlaceTag,
                 sizeof(ControlButtonTextPlacement),
                 (char *) &thePlacement)) != noErr) {
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,
                     "SetControlData BevelButtonTextPlace failed, %d\n", err );
+#endif
         }
     }
 }
