@@ -9,15 +9,11 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXCursor.c,v 1.4 2003/02/11 07:26:18 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXCursor.c,v 1.4.2.4 2006/07/20 06:26:45 das Exp $
  */
 
-#include "tkPort.h"
-#include "tkInt.h"
 #include "tkMacOSXInt.h"
 
-#include <Carbon/Carbon.h>
-
 /*
  * There are three different ways to set the cursor on the Mac.
  * The default theme cursors (listed in cursorNames below),
@@ -41,7 +37,7 @@ typedef struct {
     int type;                        /* Type of Mac cursor: for theme cursors
                                       * this is the theme cursor constant,
                                       * otherwise one of crsr or CURS */
-    unsigned int count;              /* For animating cursors, the count for the
+    int count;                       /* For animating cursors, the count for the
                                         cursor. */
 } TkMacOSXCursor;
 
@@ -370,9 +366,9 @@ TkMacOSXInstallCursor(
         if (cursor) {
             SetCursor(*cursor);
         } else {
-            /*
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"Resize cursor failed, %d\n", ResError());
-             */
+#endif
         }
     } else if (macCursorPtr == NULL) {
         SetThemeCursor(kThemeArrowCursor);
@@ -431,9 +427,14 @@ TkpSetCursor(
     }
     
     if (cursor == None) {
-        if (gCurrentCursor == NULL) {
-            cursorChanged = 0;
-        }
+        /*  
+         * This is a little tricky.  We can't really tell whether
+         * gCurrentCursor is NULL because it was NULL last time around
+         * or because we just freed the current cursor.  So if the input
+         * cursor is NULL, we always need to reset it, we can't trust the
+         * cursorChanged logic.
+         */
+         
         gCurrentCursor = NULL;
     } else {
         if (gCurrentCursor == (TkMacOSXCursor *) cursor) {
