@@ -4,7 +4,11 @@
  *	Implementation of Macintosh specific functions for debugging MacOS events,
  *      regions, etc...
  *
- *      Copyright 2001, Apple Computer, Inc.
+ * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2006 Daniel A. Steffen <das@users.sourceforge.net>
+ *
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
  *      The following terms apply to all files originating from Apple
  *      Computer, Inc. ("Apple") and associated with the software
@@ -50,7 +54,7 @@
  *      software in accordance with the terms specified in this
  *      license.
  *
- * RCS: @(#) $Id: tkMacOSXDebug.c,v 1.2.2.5 2006/01/10 05:38:20 das Exp $
+ * RCS: @(#) $Id: tkMacOSXDebug.c,v 1.2.2.8 2006/07/20 06:26:45 das Exp $
  */
 
 #include "tkMacOSXInt.h"
@@ -249,7 +253,7 @@ static MyEventName classicEventNames [] = {
  { 0, NULL }
 };
 
-char * 
+MODULE_SCOPE char * 
 CarbonEventToAscii(EventRef eventRef, char * buf)
 {     
     EventClass eventClass;
@@ -293,7 +297,7 @@ CarbonEventToAscii(EventRef eventRef, char * buf)
     return iBuf;
 }
 
-char *
+MODULE_SCOPE char *
 CarbonEventKindToAscii(EventRef eventRef, char * buf )
 {     
    EventClass eventClass;
@@ -329,7 +333,7 @@ CarbonEventKindToAscii(EventRef eventRef, char * buf )
      return buf;
 }
 
-char *
+MODULE_SCOPE char *
 ClassicEventToAscii(EventRecord * eventPtr, char * buf )
 {
     MyEventName     * names = NULL;
@@ -359,14 +363,14 @@ ClassicEventToAscii(EventRecord * eventPtr, char * buf )
  
 }
 
-void
+MODULE_SCOPE void
 printPoint(char * tag, Point * p )
 {
     fprintf(stderr,"%s %4d %4d\n",
         tag,p->h,p->v );
 }
 
-void
+MODULE_SCOPE void
 printRect(char * tag, Rect * r )
 {
     fprintf(stderr,"%s %4d %4d %4d %4d (%dx%d)\n",
@@ -374,7 +378,7 @@ printRect(char * tag, Rect * r )
         r->right - r->left + 1, r->bottom - r->top + 1);
 }
 
-void
+MODULE_SCOPE void
 printRegion(char * tag, RgnHandle rgn )
 {
     Rect r;
@@ -382,7 +386,7 @@ printRegion(char * tag, RgnHandle rgn )
     printRect(tag,&r);
 }
 
-void
+MODULE_SCOPE void
 printWindowTitle(char * tag, WindowRef window )
 {
     Str255 title;
@@ -410,7 +414,7 @@ static MsgName msgNames [] = {
     { -1, NULL }
 };
 
-char *
+MODULE_SCOPE char *
 TkMacOSXMenuMessageToAscii(int msg, char * s)
 {
     MsgName * msgNamePtr;
@@ -438,7 +442,7 @@ static MsgName trackingNames [] = {
     { -1, NULL }
 };
 
-char *
+MODULE_SCOPE char *
 MouseTrackingResultToAscii(MouseTrackingResult r, char * buf) 
 {
     MsgName * namePtr;
@@ -480,7 +484,7 @@ MouseTrackingResultToAscii(MouseTrackingResult r, char * buf)
  *----------------------------------------------------------------------
  */
 
-void *
+MODULE_SCOPE void *
 TkMacOSXGetNamedDebugSymbol(const char* module, const char* symbol)
 {
     void* addr = TkMacOSXGetNamedSymbol(module, symbol);
@@ -561,7 +565,7 @@ TkMacOSXGetNamedDebugSymbol(const char* module, const char* symbol)
 			strx = sym->n_un.n_strx;
 			if ((sym->n_type & N_TYPE) == N_SECT &&
 				sym->n_sect == txtsectx &&
-				strx > 0 && strx < strsize &&
+				strx > 0 && (uint32_t) strx < strsize &&
 				strcmp(strings + strx, symbol) == 0) {
 			    addr = (void*) sym->n_value + slide;
 			    break;
