@@ -49,15 +49,15 @@
  *      acting in its behalf permission to use and distribute the
  *      software in accordance with the terms specified in this
  *      license.
+ *
+ * RCS: @(#) $Id: tkMacOSXEntry.c,v 1.2.2.7 2006/09/10 17:07:36 das Exp $
  */
 
-#include "tk.h"
-#include "tkInt.h"
 #include "tkMacOSXInt.h"
 #include "tkMacOSXDefault.h"
 #include "tkEntry.h"
 
-ThemeButtonKind ComputeIncDecParameters (int height, int *width);
+static ThemeButtonKind ComputeIncDecParameters (int height, int *width);
 
 /*
  *--------------------------------------------------------------
@@ -76,7 +76,7 @@ ThemeButtonKind ComputeIncDecParameters (int height, int *width);
  *
  *--------------------------------------------------------------
  */
-ThemeButtonKind
+static ThemeButtonKind
 ComputeIncDecParameters (int height, int *width)
 {
     static int version = 0;
@@ -151,7 +151,7 @@ TkpDrawEntryBorderAndFocus(Entry *entryPtr, Drawable d, int isSpinbox)
     GC bgGC;
     Tk_Window tkwin = entryPtr->tkwin;
     ThemeDrawState drawState;
-    int oldWidth;
+    int oldWidth = 0;
 
     /* 
      * I use 6 as the borderwidth.  2 of the 5 go into the actual frame the
@@ -161,7 +161,7 @@ TkpDrawEntryBorderAndFocus(Entry *entryPtr, Drawable d, int isSpinbox)
      
     if (entryPtr->borderWidth != MAC_OSX_ENTRY_BORDER 
             || entryPtr->highlightWidth != MAC_OSX_FOCUS_WIDTH
-            ||entryPtr->relief != MAC_OSX_ENTRY_RELEIF) {
+            || entryPtr->relief != MAC_OSX_ENTRY_RELIEF) {
         return 0;
     }
     
