@@ -5,11 +5,12 @@
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2005-2006 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXEvent.c,v 1.3.2.6 2006/04/11 10:22:57 das Exp $
+ * RCS: @(#) $Id: tkMacOSXEvent.c,v 1.3.2.8 2006/05/12 18:17:55 das Exp $
  */
 
 #include "tkMacOSXInt.h"
@@ -33,7 +34,7 @@
  *----------------------------------------------------------------------
  */
 
-void
+MODULE_SCOPE void
 TkMacOSXFlushWindows ()
 {
     WindowRef wRef = GetWindowList();
@@ -70,7 +71,7 @@ TkMacOSXFlushWindows ()
  *----------------------------------------------------------------------
  */
 
-int  
+MODULE_SCOPE int  
 TkMacOSXProcessEvent(TkMacOSXEvent * eventPtr, MacEventStatus * statusPtr)
 {
     switch (eventPtr->eClass) {
@@ -123,7 +124,7 @@ TkMacOSXProcessEvent(TkMacOSXEvent * eventPtr, MacEventStatus * statusPtr)
  *----------------------------------------------------------------------
  */
 
-int
+MODULE_SCOPE int
 TkMacOSXProcessMenuEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 {
     int		    menuContext;
@@ -177,7 +178,7 @@ TkMacOSXProcessMenuEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
  *----------------------------------------------------------------------
  */
 
-int
+MODULE_SCOPE int
 TkMacOSXProcessCommandEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 {
     HICommand	    command;
@@ -228,6 +229,7 @@ TkMacOSXProcessCommandEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 				command.menu.menuItemIndex);
 		    }
 		}
+		statusPtr->stopProcessing = 1;
 		return 1;
 	    }
 	}
@@ -253,7 +255,7 @@ TkMacOSXProcessCommandEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
  *----------------------------------------------------------------------
  */
 
-OSStatus
+MODULE_SCOPE OSStatus
 TkMacOSXReceiveAndProcessEvent()
 {
     static EventTargetRef targetRef = NULL;
