@@ -5,21 +5,23 @@
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2006 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXSubwindows.c,v 1.2.2.4 2005/03/12 00:49:28 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXSubwindows.c,v 1.2.2.12 2006/07/24 04:45:26 das Exp $
  */
 
-#include "tkInt.h"
-#include "X11/X.h"
-#include "X11/Xlib.h"
-#include <stdio.h>
-
-#include <Carbon/Carbon.h>
 #include "tkMacOSXInt.h"
 #include "tkMacOSXDebug.h"
+#include "tkMacOSXWm.h"
+
+/*
+#ifdef	TK_MAC_DEBUG
+#define TK_MAC_DEBUG_CLIP_REGIONS
+#endif
+*/
 
 /*
  * Temporary region that can be reused.
@@ -31,8 +33,7 @@ static RgnHandle tmpRgn = NULL;
  */
 
 static void GenerateConfigureNotify (TkWindow *winPtr, int includeWin);
-static void UpdateOffsets _ANSI_ARGS_((TkWindow *winPtr, 
-        int deltaX, int deltaY));
+static void UpdateOffsets (TkWindow *winPtr, int deltaX, int deltaY);
 
 /*
  *----------------------------------------------------------------------
@@ -65,8 +66,7 @@ XDestroyWindow(
 
     TkPointerDeadWindow(macWin->winPtr);
     macWin->toplevel->referenceCount--;
-    
-    
+
     if (Tk_IsTopLevel(macWin->winPtr)) {
         WindowRef winRef;
         /*
@@ -86,7 +86,7 @@ XDestroyWindow(
                 if (TkpIsWindowFloating (winRef)) {
                     Window window;
                     
-                    window = TkMacOSXGetXWindow(FrontNonFloatingWindow());
+                    window = TkMacOSXGetXWindow(ActiveNonFloatingWindow());
                     if (window != None) {
                         TkMacOSXGenerateFocusEvent(window, 1);
                     }
@@ -145,7 +145,7 @@ XDestroyWindow(
 	    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
 	}
 	if (macWin->winPtr->parentPtr != NULL) {
-	    TkMacOSXInvalClipRgns(macWin->winPtr->parentPtr);
+	    TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr->parentPtr);
 	}
 	DisposeRgn(macWin->clipRgn);
 	DisposeRgn(macWin->aboveClipRgn);
@@ -193,13 +193,24 @@ XMapWindow(
     if (!TkMacOSXHostToplevelExists(macWin->toplevel->winPtr)) {
 	TkMacOSXMakeRealWindowExist(macWin->toplevel->winPtr);
     }
-    destPort = TkMacOSXGetDrawablePort (window);
 
     display->request++;
     macWin->winPtr->flags |= TK_MAPPED;
+    destPort = TkMacOSXGetDrawablePort (window);
     if (Tk_IsTopLevel(macWin->winPtr)) {
 	if (!Tk_IsEmbedded(macWin->winPtr)) {
-        	    ShowWindow(GetWindowFromPort(destPort));
+	    /*
+	     * XXX This should be ShowSheetWindow for kSheetWindowClass
+	     * XXX windows that have a wmPtr->master parent set.
+	     */
+	    WindowRef wRef = GetWindowFromPort(destPort);
+	    if ((TkMacOSXWindowClass(macWin->winPtr) == kSheetWindowClass)
+		    && (macWin->winPtr->wmInfoPtr->master != None)) {
+		ShowSheetWindow(wRef,
+			GetWindowFromPort(TkMacOSXGetDrawablePort(macWin->winPtr->wmInfoPtr->master)));
+	    } else {
+		ShowWindow(wRef);
+	    }
 	}
 
 	/* 
@@ -216,15 +227,13 @@ XMapWindow(
 	event.xmap.override_redirect = macWin->winPtr->atts.override_redirect;
 	Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
     } else {
-	TkMacOSXInvalClipRgns(macWin->winPtr->parentPtr);
+	/* 
+	 * Generate damage for that area of the window 
+	 */
+	SetGWorld(destPort, NULL);
+	TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr->parentPtr);
+	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
     }
-
-    /* 
-     * Generate damage for that area of the window 
-     */
-    SetGWorld (destPort, NULL);
-    TkMacOSXUpdateClipRgn(macWin->winPtr);
-    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
 }
 
 /*
@@ -253,13 +262,23 @@ XUnmapWindow(
     XEvent event;
     CGrafPtr destPort;
 
-    destPort = TkMacOSXGetDrawablePort(window);
-
     display->request++;
     macWin->winPtr->flags &= ~TK_MAPPED;
+    destPort = TkMacOSXGetDrawablePort(window);
     if (Tk_IsTopLevel(macWin->winPtr)) {
-	if (!Tk_IsEmbedded(macWin->winPtr)) {
-	    HideWindow(GetWindowFromPort(destPort));
+	if (!Tk_IsEmbedded(macWin->winPtr)
+		&& macWin->winPtr->wmInfoPtr->hints.initial_state != IconicState) {
+	    /*
+	     * XXX This should be HideSheetWindow for kSheetWindowClass
+	     * XXX windows that have a wmPtr->master parent set.
+	     */
+	    WindowRef wref = GetWindowFromPort(destPort);
+	    if ((TkMacOSXWindowClass(macWin->winPtr) == kSheetWindowClass)
+		    && (macWin->winPtr->wmInfoPtr->master != None)) {
+		HideSheetWindow(wref);
+	    } else {
+		HideWindow(wref);
+	    }
 	}
 
 	/* 
@@ -269,7 +288,7 @@ XUnmapWindow(
 	event.xany.serial = display->request;
 	event.xany.send_event = False;
 	event.xany.display = display;
-	
+
 	event.xunmap.type = UnmapNotify;
 	event.xunmap.window = window;
 	event.xunmap.event = window;
@@ -280,8 +299,8 @@ XUnmapWindow(
 	 * Generate damage for that area of the window.
 	 */
 	SetGWorld(destPort, NULL);
-	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW); /* TODO: may not be valid */
-	TkMacOSXInvalClipRgns(macWin->winPtr->parentPtr);
+	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+	TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr->parentPtr);
     }
 }
 
@@ -311,14 +330,14 @@ XResizeWindow(
 {
     MacDrawable *macWin = (MacDrawable *) window;
     CGrafPtr     destPort;
+    int havePort = 1;
 
     destPort = TkMacOSXGetDrawablePort(window);
     if (destPort == NULL) {
-	return;
+	havePort = 0;
     }
 
     display->request++;
-    SetPort( destPort);
     if (Tk_IsTopLevel(macWin->winPtr)) {
 	if (!Tk_IsEmbedded(macWin->winPtr)) {
 	    /* 
@@ -326,10 +345,13 @@ XResizeWindow(
 	     * region.  It is currently assumed that Tk will need
 	     * to completely redraw anway.
 	     */
-	    SizeWindow(GetWindowFromPort(destPort),
-		    (short) width, (short) height, false);
-	    TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
-	    TkMacOSXInvalClipRgns(macWin->winPtr);
+            if (havePort) {
+                SetPort(destPort);
+	        SizeWindow(GetWindowFromPort(destPort),
+		        (short) width, (short) height, false);
+	        TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
+	        TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr);
+            }
 	} else {
 	    int deltaX, deltaY;
 	    
@@ -344,9 +366,11 @@ XResizeWindow(
 	    if (contWinPtr != NULL) {
 	        MacDrawable *macParent = contWinPtr->privatePtr;
 
-		TkMacOSXInvalClipRgns(macParent->winPtr);	
-		TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
-		
+                if (havePort) {
+                    SetPort(destPort);
+		    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+		    TkMacOSXInvalClipRgns((Tk_Window) macParent->winPtr);	
+		}
 		deltaX = macParent->xOff +
 		    macWin->winPtr->changes.x - macWin->xOff;
 		deltaY = macParent->yOff +
@@ -376,9 +400,11 @@ XResizeWindow(
 	    return; /* TODO: Probably should be a panic */
 	}
 	
-	TkMacOSXInvalClipRgns(macParent->winPtr);	
-	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
-
+        if (havePort) {
+            SetPort(destPort);
+	    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+	    TkMacOSXInvalClipRgns((Tk_Window) macParent->winPtr);	
+        }
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
 
@@ -458,27 +484,28 @@ XMoveResizeWindow(
 {	
     MacDrawable * macWin = (MacDrawable *) window;
     CGrafPtr      destPort;
+    int havePort = 1;
 
     destPort   = TkMacOSXGetDrawablePort(window);
     if (destPort == NULL) {
-	return;
+	havePort = 0;
     }
 
-    SetPort( destPort);
     if (Tk_IsTopLevel(macWin->winPtr) && !Tk_IsEmbedded(macWin->winPtr)) {	
 	/* 
 	 * NOTE: we are not adding the new space to the update
 	 * region.  It is currently assumed that Tk will need
 	 * to completely redraw anway.
 	 */
+        if (havePort) {
+            SetPort( destPort);
+	    SizeWindow(GetWindowFromPort(destPort),
+		    (short) width, (short) height, false);
+	    MoveWindowStructure(GetWindowFromPort(destPort), x, y);
 	
-	SizeWindow(GetWindowFromPort(destPort),
-		(short) width, (short) height, false);
-	MoveWindowStructure(GetWindowFromPort(destPort), x, y);
-	
-	/* TODO: is the following right? */
-	TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
-	TkMacOSXInvalClipRgns(macWin->winPtr);
+	    TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
+	    TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr);
+        }
     } else {
 	int deltaX, deltaY, parentBorderwidth;
 	Rect bounds;
@@ -494,7 +521,7 @@ XMoveResizeWindow(
 	    
 	    contWinPtr = TkpGetOtherWindow(macWin->winPtr);
 	    if (contWinPtr == NULL) {
-	            panic("XMoveResizeWindow could not find container");
+		Tcl_Panic("XMoveResizeWindow could not find container");
 	    }
 	    macParent = contWinPtr->privatePtr;
 	    
@@ -509,9 +536,12 @@ XMoveResizeWindow(
 	        return; /* TODO: Probably should be a panic */
 	    }
 	}
-	        
-	TkMacOSXInvalClipRgns(macParent->winPtr);
-	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+
+	if (havePort) {
+	    SetPort( destPort);
+	    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+	    TkMacOSXInvalClipRgns((Tk_Window) macParent->winPtr);
+	}
 
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
@@ -532,8 +562,10 @@ XMoveResizeWindow(
 	    macWin->winPtr->changes.y;
 		
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
-	TkMacOSXWinBounds(macWin->winPtr, &bounds);
-	InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+        if (havePort) {
+	    TkMacOSXWinBounds(macWin->winPtr, &bounds);
+	    InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+        }
         GenerateConfigureNotify(macWin->winPtr, 0);
     }
 }
@@ -564,24 +596,26 @@ XMoveWindow(
 {
     MacDrawable *macWin = (MacDrawable *) window;
     CGrafPtr  destPort;
+    int havePort = 1;
 
     destPort   = TkMacOSXGetDrawablePort(window);
     if (destPort == NULL) {
-	return;
+	havePort = 0;
     }
 
-    SetPort( destPort);
     if (Tk_IsTopLevel(macWin->winPtr) && !Tk_IsEmbedded(macWin->winPtr)) {
 	/* 
 	 * NOTE: we are not adding the new space to the update
 	 * region.  It is currently assumed that Tk will need
 	 * to completely redraw anway.
 	 */
-	MoveWindowStructure( GetWindowFromPort(destPort), x, y);
+        if (havePort) { 
+            SetPort(destPort);
+	    MoveWindowStructure( GetWindowFromPort(destPort), x, y);
 
-	/* TODO: is the following right? */
-	TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
-	TkMacOSXInvalClipRgns(macWin->winPtr);
+	    TkMacOSXInvalidateWindow(macWin, TK_WINDOW_ONLY);
+	    TkMacOSXInvalClipRgns((Tk_Window) macWin->winPtr);
+        }
     } else {
 	int deltaX, deltaY, parentBorderwidth;
 	Rect bounds;
@@ -597,7 +631,7 @@ XMoveWindow(
 	    
 	    contWinPtr = TkpGetOtherWindow(macWin->winPtr);
 	    if (contWinPtr == NULL) {
-	            panic("XMoveWindow could not find container");
+		Tcl_Panic("XMoveWindow could not find container");
 	    }
 	    macParent = contWinPtr->privatePtr;
 	    
@@ -612,9 +646,12 @@ XMoveWindow(
 	    }
 	}
 
-	TkMacOSXInvalClipRgns(macParent->winPtr);
-	TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
-
+        if (havePort) {
+            SetPort(destPort);
+	    TkMacOSXInvalidateWindow(macWin, TK_PARENT_WINDOW);
+	    TkMacOSXInvalClipRgns((Tk_Window) macParent->winPtr);
+        }
+        
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
 	
@@ -634,8 +671,10 @@ XMoveWindow(
 	    macWin->winPtr->changes.y;
 		
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
-	TkMacOSXWinBounds(macWin->winPtr, &bounds);
-	InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+	if (havePort) {
+            TkMacOSXWinBounds(macWin->winPtr, &bounds);
+	    InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+        }
 	GenerateConfigureNotify(macWin->winPtr, 0);
     }
 }
@@ -671,6 +710,7 @@ XRaiseWindow(
     }
 }
 
+#if 0
 /*
  *----------------------------------------------------------------------
  *
@@ -701,6 +741,7 @@ XLowerWindow(
     	/* TODO: this should generate damage */
     }
 }
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -756,7 +797,7 @@ XConfigureWindow(
 	destPort = TkMacOSXGetDrawablePort(w);
 	if (destPort != NULL) {
 	    SetPort( destPort);
-	    TkMacOSXInvalClipRgns(winPtr->parentPtr);
+	    TkMacOSXInvalClipRgns((Tk_Window) winPtr->parentPtr);
 	    TkMacOSXWinBounds(winPtr, &bounds);
 	    InvalWindowRect(GetWindowFromPort(destPort),&bounds);
 	}
@@ -796,42 +837,86 @@ TkMacOSXUpdateClipRgn(
     if (winPtr == NULL) {
 	return;
     }
-    
-    if (winPtr->privatePtr->flags & TK_CLIP_INVALID) {
-	rgn = winPtr->privatePtr->aboveClipRgn;
-	if (tmpRgn == NULL) {
-	    tmpRgn = NewRgn();
-	}
-	
-	/* 
-	 * Start with a region defined by the window bounds.  
-	 */
 
-        x = winPtr->privatePtr->xOff;
-        y = winPtr->privatePtr->yOff;
-        SetRectRgn(rgn, (short) x, (short) y,
-	    (short) (winPtr->changes.width  + x), 
-	    (short) (winPtr->changes.height + y));
-	    
-	/* 
-	 * Clip away the area of any windows that may obscure this
-	 * window.  
-	 * For a non-toplevel window, first, clip to the parents visable
-	 * clip region.
-	 * Second, clip away any siblings that are higher in the
-	 * stacking order.
-	 * For an embedded toplevel, just clip to the container's visible
-	 * clip region.  Remember, we only allow one contained window 
-	 * in a frame, and don't support any other widgets in the frame either.
-	 * This is not currently enforced, however.
-	 */
-	
-	if (!Tk_IsTopLevel(winPtr)) { 
-	    TkMacOSXUpdateClipRgn(winPtr->parentPtr);
-	    SectRgn(rgn, 
-		    winPtr->parentPtr->privatePtr->aboveClipRgn, rgn);
-				
-	    win2Ptr = winPtr->nextPtr;
+    if (winPtr->privatePtr && winPtr->privatePtr->flags & TK_CLIP_INVALID) {
+	if (Tk_IsMapped(winPtr)) {
+	    rgn = winPtr->privatePtr->aboveClipRgn;
+	    if (tmpRgn == NULL) {
+		tmpRgn = NewRgn();
+	    }
+
+	    /* 
+	     * Start with a region defined by the window bounds.
+	     */
+
+	    x = winPtr->privatePtr->xOff;
+	    y = winPtr->privatePtr->yOff;
+	    SetRectRgn(rgn, (short) x, (short) y,
+		(short) (winPtr->changes.width  + x), 
+		(short) (winPtr->changes.height + y));
+
+	    /* 
+	     * Clip away the area of any windows that may obscure this
+	     * window.
+	     * For a non-toplevel window, first, clip to the parents visible
+	     * clip region.
+	     * Second, clip away any siblings that are higher in the
+	     * stacking order.
+	     * For an embedded toplevel, just clip to the container's visible
+	     * clip region.  Remember, we only allow one contained window
+	     * in a frame, and don't support any other widgets in the frame
+	     * either. This is not currently enforced, however.
+	     */
+
+	    if (!Tk_IsTopLevel(winPtr)) { 
+		TkMacOSXUpdateClipRgn(winPtr->parentPtr);
+		SectRgn(rgn, 
+			winPtr->parentPtr->privatePtr->aboveClipRgn, rgn);
+
+		win2Ptr = winPtr->nextPtr;
+		while (win2Ptr != NULL) {
+		    if (Tk_IsTopLevel(win2Ptr) || !Tk_IsMapped(win2Ptr)) {
+			win2Ptr = win2Ptr->nextPtr;
+			continue;
+		    }
+		    x = win2Ptr->privatePtr->xOff;
+		    y = win2Ptr->privatePtr->yOff;
+		    SetRectRgn(tmpRgn, (short) x, (short) y,
+			    (short) (win2Ptr->changes.width  + x), 
+			    (short) (win2Ptr->changes.height + y));
+		    DiffRgn(rgn, tmpRgn, rgn);
+
+		    win2Ptr = win2Ptr->nextPtr;
+		}
+	    } else if (Tk_IsEmbedded(winPtr)) {
+		TkWindow *contWinPtr = TkpGetOtherWindow(winPtr);
+
+		if (contWinPtr != NULL) {
+		    TkMacOSXUpdateClipRgn(contWinPtr);
+		    SectRgn(rgn, 
+			    contWinPtr->privatePtr->aboveClipRgn, rgn);
+		} else if (gMacEmbedHandler != NULL) {
+		    gMacEmbedHandler->getClipProc((Tk_Window) winPtr, tmpRgn);
+		    SectRgn(rgn, tmpRgn, rgn);
+		}
+
+		/*
+		 * NOTE: Here we should handle out of process embedding.
+		 */
+
+	    }
+
+	    /* 
+	     * The final clip region is the aboveClip region (or visible
+	     * region) minus all the children of this window.
+	     * Alternatively, if the window is a container, we must also 
+	     * subtract the region of the embedded window.
+	     */
+
+	    rgn = winPtr->privatePtr->clipRgn;
+	    CopyRgn(winPtr->privatePtr->aboveClipRgn, rgn);
+
+	    win2Ptr = winPtr->childList;
 	    while (win2Ptr != NULL) {
 		if (Tk_IsTopLevel(win2Ptr) || !Tk_IsMapped(win2Ptr)) {
 		    win2Ptr = win2Ptr->nextPtr;
@@ -843,75 +928,61 @@ TkMacOSXUpdateClipRgn(
 			(short) (win2Ptr->changes.width  + x), 
 			(short) (win2Ptr->changes.height + y));
 		DiffRgn(rgn, tmpRgn, rgn);
-							  
+
 		win2Ptr = win2Ptr->nextPtr;
 	    }
-	} else if (Tk_IsEmbedded(winPtr)) {
-            TkWindow *contWinPtr;
-        
-	    contWinPtr = TkpGetOtherWindow(winPtr);
-    	     
-    	    if (contWinPtr != NULL) {
- 	        TkMacOSXUpdateClipRgn(contWinPtr);
-	        SectRgn(rgn, 
-		        contWinPtr->privatePtr->aboveClipRgn, rgn);
-   	    } else if (gMacEmbedHandler != NULL) {
-   	        gMacEmbedHandler->getClipProc((Tk_Window) winPtr, tmpRgn);
-   	        SectRgn(rgn, tmpRgn, rgn);
-   	    }
-	    
+
+	    if (Tk_IsContainer(winPtr)) {
+		win2Ptr = TkpGetOtherWindow(winPtr);
+		if (win2Ptr != NULL) {
+		    if (Tk_IsMapped(win2Ptr)) {
+			x = win2Ptr->privatePtr->xOff;
+			y = win2Ptr->privatePtr->yOff;
+			SetRectRgn(tmpRgn, (short) x, (short) y,
+				(short) (win2Ptr->changes.width  + x), 
+				(short) (win2Ptr->changes.height + y));
+			DiffRgn(rgn, tmpRgn, rgn);
+		    }
+		} 
+
+		/*
+		 * NOTE: Here we should handle out of process embedding.
+		 */
+
+	    }
+	} else {
 	    /*
-	     * NOTE: Here we should handle out of process embedding.
+	     * An unmapped window has empty clip regions to prevent any
+	     * (erroneous) drawing into it or its children from becoming
+	     * visible. [Bug 940117]
 	     */
-		    
-	}
-	
-	/* 
-	 * The final clip region is the aboveClip region (or visable
-	 * region) minus all the children of this window.
-	 * Alternatively, if the window is a container, we must also 
-	 * subtract the region of the embedded window.
-	 */
-	 
-	rgn = winPtr->privatePtr->clipRgn;
-	CopyRgn(winPtr->privatePtr->aboveClipRgn, rgn);
-		
-	win2Ptr = winPtr->childList;
-	while (win2Ptr != NULL) {
-	    if (Tk_IsTopLevel(win2Ptr) || !Tk_IsMapped(win2Ptr)) {
-		win2Ptr = win2Ptr->nextPtr;
-		continue;
-	    }
-	    x = win2Ptr->privatePtr->xOff;
-	    y = win2Ptr->privatePtr->yOff;
-	    SetRectRgn(tmpRgn, (short) x, (short) y,
-		    (short) (win2Ptr->changes.width  + x), 
-		    (short) (win2Ptr->changes.height + y));
-	    DiffRgn(rgn, tmpRgn, rgn);
-							  
-	    win2Ptr = win2Ptr->nextPtr;
-	}
-	
-	if (Tk_IsContainer(winPtr)) {
-	    win2Ptr = TkpGetOtherWindow(winPtr);
-	    if (win2Ptr != NULL) {
-		if (Tk_IsMapped(win2Ptr)) {
-		    x = win2Ptr->privatePtr->xOff;
-		    y = win2Ptr->privatePtr->yOff;
-		    SetRectRgn(tmpRgn, (short) x, (short) y,
-			    (short) (win2Ptr->changes.width  + x), 
-			    (short) (win2Ptr->changes.height + y));
-		    DiffRgn(rgn, tmpRgn, rgn);
+
+	    if (!Tk_IsTopLevel(winPtr)) { 
+		TkMacOSXUpdateClipRgn(winPtr->parentPtr);
+	    } else if (Tk_IsEmbedded(winPtr)) {
+		TkWindow *contWinPtr = TkpGetOtherWindow(winPtr);
+
+		if (contWinPtr != NULL) {
+		    TkMacOSXUpdateClipRgn(contWinPtr);
 		}
-	    } 
-	    
-	    /*
-	     * NOTE: Here we should handle out of process embedding.
-	     */
-		    
+	    }
+	    SetEmptyRgn(winPtr->privatePtr->aboveClipRgn);
+	    SetEmptyRgn(winPtr->privatePtr->clipRgn);
 	}
-		
+
 	winPtr->privatePtr->flags &= ~TK_CLIP_INVALID;
+
+#if defined(TK_MAC_DEBUG) && defined(TK_MAC_DEBUG_CLIP_REGIONS)
+	TkMacOSXInitNamedDebugSymbol(HIToolbox, int, QDDebugFlashRegion,
+				     CGrafPtr port, RgnHandle region);
+	if (QDDebugFlashRegion) {
+	    MacDrawable *macDraw = (MacDrawable *) winPtr->privatePtr;
+	    CGrafPtr grafPtr = TkMacOSXGetDrawablePort((Drawable) macDraw);
+	    /* Carbon-internal region flashing SPI (c.f. Technote 2124) */
+	    QDDebugFlashRegion(grafPtr, macDraw->clipRgn);
+	}
+#endif /* TK_MAC_DEBUG_CLIP_REGIONS */
+
     }
 }
 
@@ -921,7 +992,7 @@ TkMacOSXUpdateClipRgn(
  * TkMacOSXVisableClipRgn --
  *
  *	This function returnd the Macintosh cliping region for the 
- *	given window.  A NULL Rgn means the window is not visable.
+ *	given window.  A NULL Rgn means the window is not visible.
  *
  * Results:
  *	The region.
@@ -971,13 +1042,26 @@ TkMacOSXInvalidateWindow(
 
     grafPtr = TkMacOSXGetDrawablePort((Drawable)macWin);
     windowRef = GetWindowFromPort(grafPtr);
-    
+
+    if (macWin->flags & TK_CLIP_INVALID) {
+	TkMacOSXUpdateClipRgn(macWin->winPtr);
+    }
     if (flag == TK_WINDOW_ONLY) {
-	InvalWindowRgn(windowRef,macWin->clipRgn);
+	InvalWindowRgn(windowRef, macWin->clipRgn);
     } else {
 	if (!EmptyRgn(macWin->aboveClipRgn)) {
-	    InvalWindowRgn(windowRef,macWin->aboveClipRgn);
+	    InvalWindowRgn(windowRef, macWin->aboveClipRgn);
 	}
+
+#if defined(TK_MAC_DEBUG) && defined(TK_MAC_DEBUG_CLIP_REGIONS)
+	TkMacOSXInitNamedDebugSymbol(HIToolbox, int, QDDebugFlashRegion,
+				     CGrafPtr port, RgnHandle region);
+	if (QDDebugFlashRegion) {
+	    /* Carbon-internal region flashing SPI (c.f. Technote 2124) */
+	    QDDebugFlashRegion(grafPtr, macWin->aboveClipRgn);
+	}
+#endif /* TK_MAC_DEBUG_CLIP_REGIONS */
+
     }
 }
 
@@ -1122,7 +1206,7 @@ TkMacOSXGetRootControl(
  *	This function invalidates the clipping regions for a given
  *	window and all of its children.  This function should be
  *	called whenever changes are made to subwindows that would
- *	effect the size or position of windows.
+ *	affect the size or position of windows.
  *
  * Results:
  *	None.
@@ -1136,48 +1220,51 @@ TkMacOSXGetRootControl(
 
 void
 TkMacOSXInvalClipRgns(
-    TkWindow *winPtr)
+    Tk_Window tkwin)
 {
+    TkWindow *winPtr = (TkWindow *) tkwin;
     TkWindow *childPtr;
-	
+
     /* 
      * If already marked we can stop because all 
      * decendants will also already be marked.
      */
-    if (winPtr->privatePtr->flags & TK_CLIP_INVALID) {
+    if (!winPtr->privatePtr || winPtr->privatePtr->flags & TK_CLIP_INVALID) {
 	return;
     }
-	
+
     winPtr->privatePtr->flags |= TK_CLIP_INVALID;
-	
+    SetEmptyRgn(winPtr->privatePtr->aboveClipRgn);
+    SetEmptyRgn(winPtr->privatePtr->clipRgn);
+
     /* 
      * Invalidate clip regions for all children & 
      * their decendants - unless the child is a toplevel.
      */
     childPtr = winPtr->childList;
-    while (childPtr != NULL) {
-	if (!Tk_IsTopLevel(childPtr) && Tk_IsMapped(childPtr)) {
-	    TkMacOSXInvalClipRgns(childPtr);
+    while (childPtr) {
+	if (!Tk_IsTopLevel(childPtr)) {
+	    TkMacOSXInvalClipRgns((Tk_Window) childPtr);
 	}
 	childPtr = childPtr->nextPtr;
     }
-    
+
     /*
      * Also, if the window is a container, mark its embedded window
      */
-     
+
     if (Tk_IsContainer(winPtr)) {
 	childPtr = TkpGetOtherWindow(winPtr);
 
-	if (childPtr != NULL && Tk_IsMapped(childPtr)) {
-	    TkMacOSXInvalClipRgns(childPtr);
+	if (childPtr) {
+	    TkMacOSXInvalClipRgns((Tk_Window) childPtr);
 	}
-	
+
 	/*
 	 * NOTE: Here we should handle out of process embedding.
 	 */
-		    	
-    }     	    
+
+    }
 }
 
 /*
@@ -1210,7 +1297,8 @@ TkMacOSXWinBounds(
 	    winPtr->changes.width);
     bounds->bottom = (short) (winPtr->privatePtr->yOff +
 	    winPtr->changes.height);
-}
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1330,7 +1418,7 @@ Tk_GetPixmap(
 	err = NewGWorld(&gWorld, depth, &bounds, NULL, NULL, useTempMem);
     }
     if (err != noErr) {
-        panic("Out of memory: NewGWorld failed in Tk_GetPixmap");
+        Tcl_Panic("Out of memory: NewGWorld failed in Tk_GetPixmap");
     }
 
     /*
